import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass

# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
import time

# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_uprn(user_uprn)
        check_postcode(user_postcode)

        bin_data_dict = {"bins": []}
        collections = []
        web_driver = kwargs.get("web_driver")

        data = {"bins": []}

        # Get our initial session running
        driver = create_webdriver(web_driver)
        driver.get(kwargs.get("url"))

        wait = WebDriverWait(driver, 30)
        postcode = wait.until(
            EC.presence_of_element_located((By.XPATH, '//*[@id="pPostcode"]'))
        )

        postcode.send_keys(user_postcode)
        postcode_search_btn = wait.until(
            EC.element_to_be_clickable((By.CLASS_NAME, 'searchbox_submit'))
        )     
        postcode_search_btn.send_keys(Keys.ENTER)
        # Wait for the 'Select your property' dropdown to appear and select the first result
        dropdown = wait.until(EC.element_to_be_clickable((By.ID, "uprn")))

        # Create a 'Select' for it, then select the first address in the list
        # (Index 0 is "Make a selection from the list")
        dropdownSelect = Select(dropdown)
        dropdownSelect.select_by_value(str(user_uprn))
        checkbox = wait.until(EC.presence_of_element_located((By.ID, 'gdprTerms')))
        checkbox.send_keys(Keys.SPACE)
        get_bin_data_btn = wait.until(
            EC.element_to_be_clickable((By.CLASS_NAME, 'searchbox_submit'))
        )     
        get_bin_data_btn.send_keys(Keys.ENTER)
        # Make a BS4 object
        results = wait.until(
            EC.presence_of_element_located((By.ID, 'collection'))
        )
        soup = BeautifulSoup(driver.page_source, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        # Get collections
        row_index = 0
        for row in soup.find("table", {"id": "collection"}).find_all("tr"):
            # Skip headers row
            if row_index < 1:
                row_index += 1
                continue
            else:
                # Get bin info
                bin_info = row.find_all("td")
                # Get the bin type
                bin_type = bin_info[0].find("strong").get_text(strip=True)
                # Get the collection date
                collection_date = ""
                for p in bin_info[2].find_all("p"):
                    if "your next collection" in p.get_text(strip=True):
                        collection_date = datetime.strptime(
                            " ".join(p.get_text(strip=True).replace("will be your next collection.", "").split()),
                            "%A %d %B %Y"
                        )

                if collection_date != "":
                    # Append the bin type and date to the data dict
                    dict_data = {
                        "type": bin_type,
                        "collectionDate": collection_date.strftime(date_format)
                    }
                    data["bins"].append(dict_data)

                row_index += 1

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
import time
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        data = {"bins": []}

        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")
        # Create Selenium webdriver
        page = f"https://webapp.halton.gov.uk/PublicWebForms/WasteServiceSearchv1.aspx"

        driver = create_webdriver(web_driver)
        driver.get(page)

        # If you bang in the house number (or property name) and postcode in the box it should find your property

        #iframe_presense = WebDriverWait(driver, 30).until(
        #    EC.presence_of_element_located((By.ID, "fillform-frame-1"))
        #)

        #driver.switch_to.frame(iframe_presense)
        wait = WebDriverWait(driver, 60)

        inputElement_property = wait.until(
            EC.element_to_be_clickable((By.NAME, "ctl00$ContentPlaceHolder1$txtProperty"))
        )
        inputElement_property.send_keys(user_paon)

        inputElement_postcodesearch = wait.until(
            EC.element_to_be_clickable((By.NAME, "ctl00$ContentPlaceHolder1$txtPostcode"))
        )
        inputElement_postcodesearch.send_keys(user_postcode)
        time.sleep(1)
        wait.until(EC.frame_to_be_available_and_switch_to_it((By.CSS_SELECTOR,"iframe[name^='a-'][src^='https://www.google.com/recaptcha/api2/anchor?']")))
        wait.until(EC.element_to_be_clickable((By.XPATH, "//span[@id='recaptcha-anchor']"))).click()
        time.sleep(5)
        driver.switch_to.default_content() 
        search_btn = wait.until(
            EC.element_to_be_clickable((By.XPATH, '//*[@id="ContentPlaceHolder1_btnSearch"]'))
        )
        search_btn.click()
        soup = BeautifulSoup(driver.page_source, features="html.parser")

        # Find all tab panels within the collectionTabs
        # Find all anchor elements within the collectionTabs
        anchor_elements = soup.select('#collectionTabs a.ui-tabs-anchor')

        for anchor in anchor_elements:
            # Extract the type of waste from the anchor text
            waste_type = anchor.text.strip()

            # Find the corresponding panel using the href attribute
            panel_id = anchor.get('href')
            panel = soup.select_one(panel_id)

            # Find all ul elements within the corresponding panel
            ul_elements = panel.find_all('ul')

            # Check if there are at least two ul elements
            if len(ul_elements) >= 2:
                # Get the second ul element and extract its li elements
                second_ul = ul_elements[1]
                li_elements = second_ul.find_all('li')

                # Extract the text content of each li element
                date_texts = [re.sub(r'[^a-zA-Z0-9,\s]', '', li.get_text(strip=True)).strip() for li in li_elements]

                for date_text in date_texts:
                    # Extracting dates from the text using simple text manipulation
                    # Assuming the dates are in the format: "Friday 15th December 2023", "Friday 22nd December 2023", etc.
                    # Parse the date string into a datetime object
                    date_string_without_ordinal = re.sub(r'(\d+)(st|nd|rd|th)', r'\1', date_text)

                    parsed_date = datetime.strptime(date_string_without_ordinal, '%A %d %B %Y')

                    # Format the datetime object into the desired format '%d/%m/%Y'
                    formatted_date = parsed_date.strftime('%d/%m/%Y')

                    # Add extracted data to the 'bins' list
                    data["bins"].append(
                        {
                            "type": waste_type.capitalize(),
                            "collectionDate": formatted_date
                        }
                    )

        return data


from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

import time
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)
        web_driver = kwargs.get("web_driver")

        data = {"bins": []}

        # Get our initial session running
        page = "https://carehomes.bolton.gov.uk/bins.aspx"

        driver = create_webdriver(web_driver)
        driver.get(page)

        # If you bang in the house number (or property name) and postcode in the box it should find your property
        wait = WebDriverWait(driver, 30)

        pc_search_box = wait.until(
            EC.presence_of_element_located((By.ID, "txtPostcode"))
        )

        pc_search_box.send_keys(user_postcode)

        pcsearch_btn = wait.until(
            EC.element_to_be_clickable((By.ID, "btnSubmit"))
        )

        pcsearch_btn.click()

        # Wait for the 'Select your property' dropdown to appear and select the first result
        dropdown = wait.until(EC.element_to_be_clickable((By.ID, "ddlAddresses")))

        dropdown_options = wait.until(
            EC.presence_of_element_located((By.XPATH, "//select/option[1]"))
        )
        time.sleep(1)
        # Create a 'Select' for it, then select the first address in the list
        # (Index 0 is "Make a selection from the list")
        dropdownSelect = Select(dropdown)
        dropdownSelect.select_by_value(str(user_uprn))
        dropdown_options = wait.until(
            EC.presence_of_element_located((By.ID, "pnlStep3"))
        )

        soup = BeautifulSoup(driver.page_source, features="html.parser")
        soup.prettify()

        collections = []

        # Find section with bins in
        sections = soup.find_all("div", {"class": "bin-info"})

        # For each bin section, get the text and the list elements
        for item in sections:
            words = item.find_next("strong").text.split()[2:4]
            bin_type = ' '.join(words).capitalize()
            date_list = item.find_all("p")
            for d in date_list:
                next_collection = datetime.strptime(d.text.strip(), "%A %d %B %Y")
                collections.append((bin_type, next_collection))

        # Sort the text and list elements by date
        ordered_data = sorted(collections, key=lambda x: x[1])

        # Put the elements into the dictionary
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data




from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

import time
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://mybexley.bexley.gov.uk/service/When_is_my_collection_day"

        data = {"bins": []}

        user_uprn = kwargs.get("uprn")
        user_paon = kwargs.get("paon") 
        user_postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")

        # Create Selenium webdriver
        driver = create_webdriver(web_driver)
        driver.get(page)

        # If you bang in the house number (or property name) and postcode in the box it should find your property

        iframe_presense = WebDriverWait(driver, 30).until(
            EC.presence_of_element_located((By.ID, "fillform-frame-1"))
        )


        driver.switch_to.frame(iframe_presense)
        wait = WebDriverWait(driver, 60)
        start_btn = wait.until(
            EC.element_to_be_clickable((By.XPATH, "//button/span[contains(text(), 'Next')]"))
        )

        start_btn.click()

        inputElement_postcodesearch = wait.until(
            EC.element_to_be_clickable((By.ID, "postcode_search"))
        )
        inputElement_postcodesearch.send_keys(user_postcode)

        find_address_btn = wait.until(
            EC.element_to_be_clickable((By.XPATH, '//*[@id="search"]'))
        )
        find_address_btn.click()

        dropdown_options = wait.until(
            EC.presence_of_element_located((By.XPATH, '//*[@id="select2-chosen-1"]'))
        )
        time.sleep(2)
        dropdown_options.click()
        time.sleep(1)
        dropdown_input = wait.until(
            EC.presence_of_element_located((By.XPATH, '//*[@id="s2id_autogen1_search"]'))
        )        
        time.sleep(1)
        dropdown_input.send_keys(user_paon)
        dropdown_input.send_keys(Keys.ENTER)

        results_found = wait.until(
            EC.presence_of_element_located((By.CLASS_NAME, "found-content"))
        )
        finish_btn = wait.until(
            EC.element_to_be_clickable((By.XPATH, "//button/span[contains(text(), 'Next')]"))
        )
        finish_btn.click()
        final_page = wait.until(
            EC.presence_of_element_located((By.CLASS_NAME, "waste-header-container"))
        )

        
        soup = BeautifulSoup(driver.page_source, features="html.parser")

        bin_fields = soup.find_all("div", class_="waste-panel-container")
        # Define your XPath

        for bin in bin_fields:

            # Extract h3 text from the current element
            h3_text = bin.find('h3', class_='container-name').get_text(strip=True) if bin.find('h3', class_='container-name') else None

            date_text = bin.find('p', class_='container-status').get_text(strip=True) if bin.find('p', class_='container-status') else None

            if h3_text and date_text:
                # Parse the date using the appropriate format
                parsed_date = datetime.strptime(date_text, "%A %d %B")
                
                # Assuming the current year is used for the collection date
                current_year = datetime.now().year
                
                # If the parsed date is in the past, assume it's for the next year
                if parsed_date < datetime.now():
                    current_year += 1

                data["bins"].append(
                    {
                        "type": h3_text,
                        "collectionDate": parsed_date.replace(year=current_year).strftime("%d/%m/%Y")
                    }
                )

        return data
